@@ -17,59 +17,123 @@ from abc import ABC
 from argparse import ArgumentParser, Namespace
 from typing import cast, List, Optional, Type, TypeVar, Union
 
-from pytorch_lightning.accelerators.accelerator import Accelerator
-from pytorch_lightning.callbacks import Callback, EarlyStopping, ModelCheckpoint, ProgressBarBase
+import torch
+from torch.optim import Optimizer
+
+from pytorch_lightning.accelerators import Accelerator
+from pytorch_lightning.callbacks import EarlyStopping, ModelCheckpoint, ProgressBarBase
+from pytorch_lightning.callbacks.base import Callback
 from pytorch_lightning.core.lightning import LightningModule
-from pytorch_lightning.core.optimizer import is_lightning_optimizer
-from pytorch_lightning.loggers.base import LightningLoggerBase
+from pytorch_lightning.core.optimizer import LightningOptimizer
+from pytorch_lightning.loggers import LightningLoggerBase
 from pytorch_lightning.loggers.tensorboard import TensorBoardLogger
+from pytorch_lightning.plugins import ParallelPlugin, PrecisionPlugin, TrainingTypePlugin
+from pytorch_lightning.trainer.connectors.accelerator_connector import AcceleratorConnector
 from pytorch_lightning.trainer.connectors.checkpoint_connector import CheckpointConnector
 from pytorch_lightning.trainer.connectors.logger_connector import LoggerConnector
-from pytorch_lightning.trainer.connectors.model_connector import ModelConnector
 from pytorch_lightning.trainer.states import TrainerState
-from pytorch_lightning.utilities import argparse_utils, HOROVOD_AVAILABLE, rank_zero_warn, TPU_AVAILABLE
+from pytorch_lightning.utilities import DeviceType, DistributedType, rank_zero_warn
+from pytorch_lightning.utilities.argparse import (
+    add_argparse_args,
+    from_argparse_args,
+    parse_argparser,
+    parse_env_variables,
+)
 from pytorch_lightning.utilities.cloud_io import get_filesystem
-from pytorch_lightning.utilities.model_utils import is_overridden
-
-if TPU_AVAILABLE:
-    import torch_xla.core.xla_model as xm
-
-if HOROVOD_AVAILABLE:
-    import horovod.torch as hvd
+from pytorch_lightning.utilities.model_helpers import is_overridden
 
 
 class TrainerProperties(ABC):
 
-    precision: int
-    logger_connector: LoggerConnector
-    _state: TrainerState
-    global_rank: int
-    fast_dev_run: Union[int, bool]
-    use_dp: bool
-    use_ddp: bool
-    use_ddp2: bool
-    model: LightningModule
-    data_parallel_device_ids: Optional[List[int]]
-    _progress_bar_callback: ProgressBarBase
-    limit_val_batches: int
     _default_root_dir: str
-    _weights_save_path: str
-    accelerator_backend: Accelerator
-    logger: LightningLoggerBase
-    model_connector: ModelConnector
-    checkpoint_connector: CheckpointConnector
-    callbacks: List[Callback]
     _lightning_optimizers = None
+    _progress_bar_callback: ProgressBarBase
+    _state: TrainerState
+    _weights_save_path: str
+
+    accelerator_connector: AcceleratorConnector
+    callbacks: List[Callback]
+    checkpoint_connector: CheckpointConnector
+    limit_val_batches: int
+    logger: LightningLoggerBase
+    logger_connector: LoggerConnector
 
     @property
-    def log_dir(self):
+    def accelerator(self) -> Accelerator:
+        return self.accelerator_connector.accelerator
+
+    @property
+    def distributed_backend(self) -> Optional[str]:
+        # for backward compatibility
+        return self.accelerator_connector.distributed_backend
+
+    @property
+    def training_type_plugin(self) -> TrainingTypePlugin:
+        return self.accelerator.training_type_plugin
+
+    @property
+    def precision_plugin(self) -> PrecisionPlugin:
+        return self.accelerator.precision_plugin
+
+    @property
+    def global_rank(self) -> int:
+        return self.accelerator.training_type_plugin.global_rank
+
+    @property
+    def local_rank(self) -> int:
+        # some training types define a local rank
+        return getattr(self.accelerator.training_type_plugin, "local_rank", 0)
+
+    @property
+    def node_rank(self) -> int:
+        # some training types define a local rank
+        return getattr(self.accelerator.training_type_plugin, "node_rank", 0)
+
+    @property
+    def world_size(self) -> int:
+        # some training types define a world size
+        return getattr(self.accelerator.training_type_plugin, "world_size", 1)
+
+    @property
+    def _distrib_type(self) -> DistributedType:
+        return self.accelerator_connector._distrib_type
+
+    @property
+    def _device_type(self) -> DeviceType:
+        return self.accelerator_connector._device_type
+
+    @property
+    def num_nodes(self) -> int:
+        return self.accelerator_connector.num_nodes
+
+    @property
+    def num_processes(self) -> int:
+        return self.accelerator_connector.num_processes
+
+    @property
+    def root_gpu(self) -> Optional[int]:
+        return self.accelerator_connector.root_gpu
+
+    @property
+    def tpu_cores(self) -> int:
+        return self.accelerator_connector.tpu_cores
+
+    @property
+    def num_gpus(self) -> int:
+        return self.accelerator_connector.num_gpus
+
+    @property
+    def data_parallel_device_ids(self) -> Optional[List[int]]:
+        return self.accelerator_connector.parallel_device_ids
+
+    @property
+    def log_dir(self) -> Optional[str]:
         if self.logger is None:
             dirpath = self.default_root_dir
         else:
             dirpath = getattr(self.logger, 'log_dir' if isinstance(self.logger, TensorBoardLogger) else 'save_dir')
 
-        if self.accelerator_backend is not None:
-            dirpath = self.accelerator_backend.broadcast(dirpath)
+        dirpath = self.accelerator.broadcast(dirpath)
         return dirpath
 
     @property
@@ -77,27 +141,27 @@ class TrainerProperties(ABC):
         return self.precision == 16
 
     @property
-    def callback_metrics(self):
+    def callback_metrics(self) -> dict:
         return self.logger_connector.callback_metrics
 
     @callback_metrics.setter
-    def callback_metrics(self, x):
+    def callback_metrics(self, x: dict) -> None:
         self.logger_connector.callback_metrics = x
 
     @property
-    def logged_metrics(self):
+    def logged_metrics(self) -> dict:
         return self.logger_connector.logged_metrics
 
     @logged_metrics.setter
-    def logged_metrics(self, x):
+    def logged_metrics(self, x: dict) -> None:
         self.logger_connector.logged_metrics = x
 
     @property
-    def progress_bar_metrics(self):
+    def progress_bar_metrics(self) -> dict:
         return self.logger_connector.progress_bar_metrics
 
     @progress_bar_metrics.setter
-    def progress_bar_metrics(self, x):
+    def progress_bar_metrics(self, x: dict) -> None:
         self.logger_connector.progress_bar_metrics = x
 
     @property
@@ -110,21 +174,21 @@ class TrainerProperties(ABC):
 
     @property
     def slurm_job_id(self) -> Optional[int]:
-        try:
-            job_id = os.environ['SLURM_JOB_ID']
-            job_id = int(job_id)
-
-            # in interactive mode, don't make logs use the same job id
-            in_slurm_interactive_mode = os.environ['SLURM_JOB_NAME'] == 'bash'
-            if in_slurm_interactive_mode:
+        job_id = os.environ.get('SLURM_JOB_ID')
+        if job_id:
+            try:
+                job_id = int(job_id)
+            except ValueError:
                 job_id = None
 
-        except Exception:
+        # in interactive mode, don't make logs use the same job id
+        in_slurm_interactive_mode = os.environ.get('SLURM_JOB_NAME') == 'bash'
+        if in_slurm_interactive_mode:
             job_id = None
         return job_id
 
     @classmethod
-    def default_attributes(cls):
+    def default_attributes(cls) -> dict:
         init_signature = inspect.signature(cls)
 
         args = {}
@@ -145,39 +209,38 @@ class TrainerProperties(ABC):
 
     @classmethod
     def from_argparse_args(cls: Type['_T'], args: Union[Namespace, ArgumentParser], **kwargs) -> '_T':
-        return argparse_utils.from_argparse_args(cls, args, **kwargs)
+        return from_argparse_args(cls, args, **kwargs)
 
     @classmethod
     def parse_argparser(cls, arg_parser: Union[ArgumentParser, Namespace]) -> Namespace:
-        return argparse_utils.parse_argparser(cls, arg_parser)
+        return parse_argparser(cls, arg_parser)
 
     @classmethod
     def match_env_arguments(cls) -> Namespace:
-        return argparse_utils.parse_env_variables(cls)
+        return parse_env_variables(cls)
 
     @classmethod
     def add_argparse_args(cls, parent_parser: ArgumentParser) -> ArgumentParser:
-        return argparse_utils.add_argparse_args(cls, parent_parser)
+        return add_argparse_args(cls, parent_parser)
 
     @property
-    def num_gpus(self) -> int:
-        gpus = self.data_parallel_device_ids
-        if gpus is None:
-            return 0
-        return len(gpus)
+    def gpus(self) -> Optional[Union[List[int], str, int]]:
+        return self.accelerator_connector.gpus
 
     @property
     def data_parallel(self) -> bool:
-        return self.use_dp or self.use_ddp or self.use_ddp2
+        return self._distrib_type in (
+            DistributedType.DP, DistributedType.DDP, DistributedType.DDP_SPAWN, DistributedType.DDP2
+        )
 
     @property
-    def progress_bar_callback(self):
+    def progress_bar_callback(self) -> Optional[ProgressBarBase]:
         return self._progress_bar_callback
 
     @property
     def progress_bar_dict(self) -> dict:
         """ Read-only for progress bar metrics. """
-        ref_model = self.get_model()
+        ref_model = self.lightning_module
         ref_model = cast(LightningModule, ref_model)
 
         standard_metrics = ref_model.get_progress_bar_dict()
@@ -188,8 +251,7 @@ class TrainerProperties(ABC):
                 f"The progress bar already tracks a metric with the name(s) '{', '.join(duplicates)}' and"
                 f" `self.log('{duplicates[0]}', ..., prog_bar=True)` will overwrite this value. "
                 f" If this is undesired, change the name or override `get_progress_bar_dict()`"
-                f" in `LightingModule`.",
-                UserWarning
+                f" in `LightingModule`.", UserWarning
             )
         all_metrics = dict(**standard_metrics)
         all_metrics.update(**logged_metrics)
@@ -203,7 +265,7 @@ class TrainerProperties(ABC):
     @property
     def enable_validation(self) -> bool:
         """ Check if we should run validation during training. """
-        model_ref = self.model_connector.get_model()
+        model_ref = self.lightning_module
         val_loop_enabled = is_overridden('validation_step', model_ref) and self.limit_val_batches > 0
         return val_loop_enabled
 
@@ -261,51 +323,94 @@ class TrainerProperties(ABC):
         """
         return [c for c in self.callbacks if isinstance(c, ModelCheckpoint)]
 
-    def save_checkpoint(self, filepath, weights_only: bool = False):
+    def save_checkpoint(self, filepath, weights_only: bool = False) -> None:
         self.checkpoint_connector.save_checkpoint(filepath, weights_only)
 
-    def get_model(self):
-        return self.model_connector.get_model()
+    @property
+    def model(self) -> torch.nn.Module:
+        """
+        The LightningModule, but possibly wrapped into DataParallel or DistributedDataParallel.
+        To access the pure LightningModule, use
+        :meth:`~pytorch_lightning.trainer.trainer.Trainer.lightning_module` instead.
+        """
+        return self.accelerator.model
+
+    @model.setter
+    def model(self, model: torch.nn.Module) -> None:
+        """
+        Setter for the model, pass-through to accelerator and plugin where the model reference is stored.
+        Used by the Tuner to reset the state of Trainer and Accelerator.
+
+        Args:
+            model: The LightningModule, possibly wrapped into DataParallel or DistributedDataParallel, depending
+                on the backend.
+        """
+        self.accelerator.model = model
 
     @property
-    def lightning_optimizers(self):
+    def lightning_optimizers(self) -> List[LightningOptimizer]:
         if self._lightning_optimizers is None:
             self.convert_to_lightning_optimizers()
         return self._lightning_optimizers
 
+    @property
+    def lightning_module(self) -> LightningModule:
+        return self.accelerator.lightning_module
+
+    @property
+    def optimizers(self) -> Optional[List[Optimizer]]:
+        return self.accelerator.optimizers
+
+    @optimizers.setter
+    def optimizers(self, new_optims: Optional[List[Optimizer]]) -> None:
+        # Necessary to rewrap optimizers to lightning
+        # They will be re-created when accessing
+        # the `lightning_optimizers` trainer property
+        self._lightning_optimizers = None
+
+        self.accelerator.optimizers = new_optims
+
+    @property
+    def lr_schedulers(self) -> Optional[list]:
+        return self.accelerator.lr_schedulers
+
+    @lr_schedulers.setter
+    def lr_schedulers(self, new_schedulers: Optional[list]) -> None:
+        self.accelerator.lr_schedulers = new_schedulers
+
+    @property
+    def optimizer_frequencies(self) -> list:
+        return self.accelerator.optimizer_frequencies
+
+    @optimizer_frequencies.setter
+    def optimizer_frequencies(self, new_freqs: list) -> None:
+        self.accelerator.optimizer_frequencies = new_freqs
+
+    @property
+    def amp_backend(self) -> Optional[str]:
+        return self.accelerator.amp_backend
+
+    @property
+    def precision(self) -> Union[str, int]:
+        return self.accelerator.precision
+
+    @property
+    def scaler(self):
+        return self.accelerator.scaler
+
+    # TODO: refactor this so that it can be done in LightningOptimizer
     def __getstate__(self):
         # remove lightning_optimizers
         self._lightning_optimizers = None
         return self.__dict__
 
-    @property
-    def require_distributed_sampler(self):
-        if self.accelerator_backend is not None:
-            return self.accelerator_backend.require_distributed_sampler
-        return self.use_ddp or self.use_ddp2 or self.use_horovod or self.use_tpu
+    def __setstate__(self, state):
+        self.__dict__ = state
 
     @property
-    def distributed_sampler_kwargs(self):
-        if self.accelerator_backend is not None:
-            return self.accelerator_backend.distributed_sampler_kwargs
-
-        if self.use_tpu:
-            kwargs = dict(num_replicas=xm.xrt_world_size(), rank=xm.get_ordinal())
-
-        elif self.use_horovod:
-            kwargs = dict(num_replicas=hvd.size(), rank=hvd.rank())
-
-        else:
-            world_size = {
-                "ddp": self.num_nodes * self.num_processes,
-                "ddp_spawn": self.num_nodes * self.num_processes,
-                "ddp2": self.num_nodes,
-                "ddp_cpu": self.num_processes * self.num_nodes
-            }
-            assert self.distributed_backend is not None
-            kwargs = dict(num_replicas=world_size[self.distributed_backend], rank=self.global_rank)
-
-        return kwargs
+    def distributed_sampler_kwargs(self) -> Optional[dict]:
+        if isinstance(self.training_type_plugin, ParallelPlugin):
+            return self.training_type_plugin.distributed_sampler_kwargs
 
 
 # Used to represent the concrete type TrainerProperties class methods are called on.
