@@ -13,6 +13,7 @@
 # limitations under the License.
 """Helper functions to help with reproducibility of models. """
 
+import logging
 import os
 import random
 from typing import Optional
@@ -20,9 +21,10 @@ from typing import Optional
 import numpy as np
 import torch
 
-from pytorch_lightning import _logger as log
 from pytorch_lightning.utilities import rank_zero_warn
 
+log = logging.getLogger(__name__)
+
 
 def seed_everything(seed: Optional[int] = None) -> int:
     """
